#ifndef ARCHitecture_DEFINED

#define ARCHitecture_DEFINED

#ifdef USE_SOLARIS
#  define USE_UNIX_TCP
#  define USE_BIG_ENDIAN
#elif USE_LINUX
#  define USE_UNIX_TCP
#  define USE_LITTLE_ENDIAN
#else
#  define USE_WINDOWS_TCP
#  define USE_LITTLE_ENDIAN
#endif

#define OS_UNIX_DIR_SEPARATOR       '/'
#define OS_WINDOWS_DIR_SEPARATOR    '\\'

#if USE_SOLARIS || USE_LINUX
#  include <errno.h>
#  define Sys_Error       errno
#ifndef  TRUE
#  define TRUE            1
#  define FALSE           0
#endif
#  define OS_DIR_SEPARATOR          OS_UNIX_DIR_SEPARATOR

typedef long long DLONG;
typedef long long _int64;
typedef unsigned long long DWORD64;
#define _ltoa ltoa

#else
#  define Sys_Error       GetLastError()
#  define OS_DIR_SEPARATOR          OS_WINDOWS_DIR_SEPARATOR

typedef _int64 DLONG;
#endif

#define ENVCRC32_SIZE     sizeof(DLONG)

#ifdef USE_BIG_ENDIAN
#include <netinet/in.h>
#  define To_Little(x)          ((((unsigned short)x)<<8) | (((unsigned short)x)>>8))
#  define To_Big(x)             ((((unsigned short)x)<<8) | (((unsigned short)x)>>8))
#  define To_Big_ETC(x)         x
#  define To_Big_UInt_ETC(x)    x
#  define To_IntLittle(x)       ((((unsigned int)x)<<24)  | (((unsigned int)x)>>24)  | ((((unsigned int)x)<<8)&0x00FF0000)  | ((((unsigned int)x)>>8) & 0x0000FF00))
//#  define To_LongLittle(x)      ((((unsigned long)x)<<24) | (((unsigned long)x)>>24) | ((((unsigned long)x)<<8)&0x00FF0000) | ((((unsigned long)x)>>8) & 0x0000FF00))
#  define To_UIntBig(x)         ((((unsigned int)x)<<24) | (((unsigned int)x)>>24) | ((((unsigned int)x)<<8)&0x00FF0000) | ((((unsigned int)x)>>8) & 0x0000FF00))
#  define Big_Ushort(x)         x

// the number has big format, take the value as big
#pragma pack (1)
struct DLONG_PTR_S
{
  DLONG value;
};
struct ULONG_PTR_S
{
  unsigned long value;
};
struct LONG_PTR_S
{
  long value;
};
struct USHORT_PTR_S
{
  unsigned short value;
};
struct SHORT_PTR_S
{
  short value;
};
struct INT_PTR_S
{
  int value;
};
struct UINT_PTR_S
{
  int value;
};
#pragma pack ()

// the number has big format, take the value as big
#  define Take_UShort(_x)         (((USHORT_PTR_S *)(_x))->value)
#  define Take_Short(_x)          (((SHORT_PTR_S *)(_x))->value)
#  define Take_UInt(_x)           (((UINT_PTR_S *)(_x))->value)
#  define Take_Int(_x)            (((INT_PTR_S *)(_x))->value)
#  define Take_ULong(_x)          (((ULONG_PTR_S *)(_x))->value)
#  define Take_Long(_x)           (((LONG_PTR_S *)(_x))->value)
#  define Take_DLong(_x)           (((DLONG_PTR_S *)(_x))->value)
#  define Take_Ptr(_x,type)       ((type)Take_ULong(_x))  // here x is supossed to be void **, or char **, and the *x is obtained

// the number has little format, and the big value is required
#  define Take_Big_UShort(_x)     ((unsigned short)((((unsigned char *)_x)[1])<<8 | (((unsigned char *)_x)[0])))
#  define Take_Big_Short(_x)      ((signed short)  ((((unsigned char *)_x)[1])<<8 | (((unsigned char *)_x)[0])))
#  define Take_Big_UInt(_x)       ((unsigned int) ((((((unsigned char *)_x)[3]) << 8 | ((unsigned char *)_x)[2])<< 8 | ((unsigned char *)_x)[1])<< 8 | ((unsigned char *)_x)[0]))
#  define Take_Big_Int(_x)        ((signed int)   ((((((unsigned char *)_x)[3]) << 8 | ((unsigned char *)_x)[2])<< 8 | ((unsigned char *)_x)[1])<< 8 | ((unsigned char *)_x)[0]))
//#  define Take_Big_ULong(_x)      ((unsigned long)((((((unsigned char *)_x)[3]) << 8 | ((unsigned char *)_x)[2])<< 8 | ((unsigned char *)_x)[1])<< 8 | ((unsigned char *)_x)[0]))
//#  define Take_Big_Long(_x)       ((signed long)  ((((((unsigned char *)_x)[3]) << 8 | ((unsigned char *)_x)[2])<< 8 | ((unsigned char *)_x)[1])<< 8 | ((unsigned char *)_x)[0]))

#  define Put_UShort(_x,y)        (((USHORT_PTR_S *)(_x))->value = y)
#  define Put_Short(_x,y)         (((SHORT_PTR_S *)(_x))->value = y)
#  define Put_UInt(_x,y)          (((UINT_PTR_S *)(_x))->value = y)
#  define Put_Int(_x,y)           (((INT_PTR_S *)(_x))->value = y)
//#  define Put_ULong(_x,y)         (((ULONG_PTR_S *)(_x))->value = y)
//#  define Put_Long(_x,y)          (((LONG_PTR_S *)(_x))->value = y)
#else
#  define To_Little(x)          x
#  define To_Big(x)             x
#  define To_Big_ETC(x)         ((((unsigned short)x)<<8) | (((unsigned short)x)>>8))
#  define To_Big_UInt_ETC(x)    ((((unsigned int)x) << 24) | ((((unsigned int)x) << 8) & 0x00FF0000) | ((((unsigned int)x) >> 8) & 0x0000FF00) | (((unsigned int)x) >> 24)) 
#  define To_IntLittle(x)       x
//#  define To_LongLittle(x)      x
#  define To_UIntBig(x)         x
#  define Big_Ushort(x)         ntohs(x)

// the number has big format, take the value as big
#  define Take_UShort(_x)       (*(unsigned short *)(_x))
#  define Take_Short(_x)        (*(short *)(_x))
#  define Take_UInt(_x)         (*(unsigned int *)(_x))
#  define Take_Int(_x)          (*(int *)(_x))
#  define Take_ULong(_x)        (*(unsigned int *)(_x))
#  define Take_Long(_x)         (*(long *)(_x))   //QUEDA COMO LONG, usar anteriores
#  define Take_DLong(_x)        (*(DLONG *)(_x))
#  define Take_Ptr(_x,type)      ((type)(*(_x)))  // here x is supossed to be void **, or char **, and the *x is obtained

// the number has little format, and the big value is required
#  define Take_Big_UShort(_x)   (*(unsigned short *)(_x))
#  define Take_Big_Short(_x)    (*(short *)(_x))
#  define Take_Big_UInt(_x)     (*(unsigned int *)(_x))
#  define Take_Big_Int(_x)      (*(signed int *)(_x))
//#  define Take_Big_ULong(_x)    (*(unsigned long *)(_x))
//#  define Take_Big_Long(_x)     (*(signed long *)(_x))

#  define Put_UShort(_x,y)      *(unsigned short *)(_x) = y
#  define Put_Short(_x,y)       *(short *)(_x) = y
#  define Put_UInt(_x,y)        *(unsigned int *)(_x) = y
#  define Put_Int(_x,y)         *(int *)(_x) = y
//#  define Put_ULong(_x,y)       *(unsigned long *)(_x) = y
//#  define Put_Long(_x,y)        *(signed long *)(_x) = y

#ifdef USE_LINUX
#ifndef VOID
#define VOID void
#endif

#define  LongToPtr( l )   ((void *)(long)l)
#define  UlongToPtr(ul)   ((void *)(unsigned long)ul)  //VERIFICAR SU USO EN MESSAGE.CPP CON COLLECT, LO MISMO CON LA OPUESTA
#define  PtrToLong(ptr)   ((long)ptr)
#define  PtrToUlong(ptr)  ((unsigned long)ptr)
#endif

#endif
#endif
