//////////////////////////////////////////////////////////////////////////////
// arrm.h
//
// Implementation of the receiving and send messages
//////////////////////////////////////////////////////////////////////////////

#ifndef _ARRM_H_
#define _ARRM_H_

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#define THMAIN          0x001
#define THCON           0x002
#define THDSCONI        0x004
#define THDSCONS        0x008
#define THGICONI        0x010
#define THGICONS        0x020
#define THBRCONI        0x040
#define THBRCONS        0x080
#define THPROC          0x100
#define THLOG           0x200
#define THREST          0x400

#include "thalert.h"

#undef GLOBAL

#ifdef  ARRM_GLOBAL_HERE
#   define GLOBAL
#else
#   define GLOBAL extern
#endif

GLOBAL unsigned int  Glb_Tot_Simulated_Collections;
GLOBAL unsigned int  Glb_Tot_Pending_Collections;

// the default number of records to maintain

#ifndef DAPI
#define CMSS_INITIAL_NUM_RECORDS  300
#else
#define CMSS_INITIAL_NUM_RECORDS  6
#endif

#ifdef DLOG
// the default number of records to wait
#define CMSS_MAX_NUM_WAITING_RECORDS 1000
// the default record length 
#define CMSS_RECORD_LEN           4000
#else
// the default record length 
#define CMSS_RECORD_LEN           2000
#endif
#define TRANS_STACK_SIZE          100

// the defulat record operating type
#define CMSS_FIFO           1
    // this type is intended for the secondary DME, usually the there is not output in the stack
#define CMSS_NOT_OUTPUT     2

#ifdef USE_SOLARIS
#  define USE_POSIX_THREADS
#elif USE_LINUX
#ifndef USE_POSIX_THREADS
#  define USE_POSIX_THREADS
#endif
#else
#  define USE_WIN_THREADS
#endif

#include "arch.h"

#ifdef DMEAPP
#define USE_TRANSACTIONS
#ifdef NO_USE_TRANSACTIONS
#  undef USE_TRANSACTIONS  
#endif
#endif

#ifdef USE_POSIX_THREADS
//  cc -mt [ flag... ] file... -lpthread [ library... ]
#include <pthread.h>
#include <semaphore.h>
#include <unistd.h>
#include <syslog.h>
#include <ctype.h>

#define _getcwd   getcwd
#define _close    close

#ifdef USE_SOLARIS
#include <thread.h>
#include <sys/priocntl.h>
#include <sys/rtpriocntl.h>

typedef thread_t                        Thread_Type;
#define Thread_Stack_Create(x,y,z,t,s)  thr_create(NULL, s, y, z, 0, &x)
#define Thread_Main_Stack_Create(x,y,z,t,s)  thr_create(NULL, s, y, z, Glb_Sol_Thread_Flags, &x)
#define Thread_Create(x,y,z,t)          thr_create(NULL, 0, y, z, 0, &x)
#define Thread_Main_Create(x,y,z,t)     thr_create(NULL, 0, y, z, Glb_Sol_Thread_Flags, &x)
#define Thread_Join(x)                  thr_join(x,NULL, NULL)
#define Thread_Sig_Set_Mask(x,y,z)      thr_sigsetmask(x, y, z)
#define Thread_End()

#elif USE_LINUX

#include <linux/limits.h>

typedef pthread_t                            Thread_Type;
#define Thread_Main_Stack_Create(x,y,z,t,s)  pthread_create(&x,&Glb_pthread_attr,y,z)
#define Thread_Stack_Create(x,y,z,t,s)         pthread_create(&x,NULL,y,z)
#define Thread_Create(x,y,z,t)                 pthread_create(&x,NULL,y,z)
#define Thread_Main_Create(x,y,z,t)          pthread_create(&x,&Glb_pthread_attr,y,z)
#define Thread_Join(x)                       pthread_join(x,NULL)
#define Thread_Sig_Set_Mask(x,y,z)           pthread_sigmask(x, y, z)
#define Thread_End()

#endif

typedef int                   Thread_Result;
typedef pthread_mutex_t       Mutex_Type;
typedef sem_t                 Semaphore_Type;
#define Thread_Is(x)          pthread_equal (pthread_self(), x)
#define Thread_Get            pthread_self
#define Thread_Status(x,y)    pthread_kill(x,0)
GLOBAL  pthread_mutexattr_t  *Glb_Mutex_Attr;
#define Mutex_Init(x)         pthread_mutex_init(&x,Glb_Mutex_Attr)
#define Mutex_Lock(x)         _LM_(#x) pthread_mutex_lock(&x)
#define Mutex_Try(x)          (pthread_mutex_trylock(&x)==0)
#define Mutex_Unlock(x)       _UM_(#x) pthread_mutex_unlock(&x)
#define _mutex_lock(x)         pthread_mutex_lock(&x)
#define _mutex_unlock(x)       pthread_mutex_unlock(&x)
#define Mutex_Destroy(x)      pthread_mutex_destroy(&x)
#define Semaphore_Init(x)     sem_init(&x,0,0)
#define Semaphore_Destroy(x)  sem_destroy(&x)
#define Semaphore_Wait(x)     sem_wait(&x)
#define Semaphore_Try_Wait(x) (sem_trywait(&x)==0)
#define Semaphore_Post(x)     sem_post(&x)
#ifdef DMDIAPP
#define UNIX_PTR              extern "C" void *
#else
#define UNIX_PTR              void *
#endif
#define MAX_PATH              PATH_MAX
#define STILL_ACTIVE          0
#define Get_LocalTime(x,y)    localtime_r(x, y)
#define Get_NextToken(x,y,z)  strtok_r(x,y,z)

#else
#include <winsock2.h>
//#include <windows.h>
#include <time.h>
#include <process.h>
#if DAPI && !VC600
typedef uintptr_t             Thread_Type;
typedef uintptr_t             Thread_Result; // this is the thread handle
#else
typedef uintptr_t             Thread_Type;
typedef uintptr_t             Thread_Result; // this is the thread handle
#endif

//#ifndef DMDIAPP
typedef int                             pid_t;
//#endif

typedef HANDLE                          Mutex_Type;
typedef HANDLE                          Semaphore_Type;
#define Thread_Is(x)                    (GetCurrentThreadId()==x)
#define Thread_Get                      GetCurrentThreadId
#define Thread_Stack_Create(x,y,z,t,s)  x=_beginthreadex(NULL,s,y,z,0,t)
#define Thread_Main_Stack_Create(x,y,z,t,s)  x=_beginthreadex(NULL,s,y,z,0,t)
#define Thread_Create(x,y,z,t)          x=_beginthreadex(NULL,0,y,z,0,t)
#define Thread_Main_Create(x,y,z,t)     x=_beginthreadex(NULL,0,y,z,0,t)
#define Thread_End()                    _endthreadex(0)
#define Thread_Join(x)                  (WaitForSingleObject((HANDLE)x, 20000));CloseHandle((HANDLE)x)  
#define Thread_Status(x,y)              !GetExitCodeThread((HANDLE)x,(unsigned int *)&y)
#define Mutex_Init(x)                   x=CreateMutex(NULL,false,NULL)
#define Mutex_Lock(x)                 _LM_(#x) WaitForSingleObject(x,INFINITE)
#define Mutex_Try(x)                  (WaitForSingleObject(x,0)==WAIT_OBJECT_0)
#define Mutex_Unlock(x)               _UM_(#x) ReleaseMutex(x)
#define _mutex_lock(x)                WaitForSingleObject(x,INFINITE)
#define _mutex_unlock(x)              ReleaseMutex(x)
#define Mutex_Destroy(x)              CloseHandle(x)
#define Semaphore_Init(x)             x=CreateSemaphore(NULL,0,0xFFFFFFF,NULL)
#define Semaphore_Destroy(x)          CloseHandle(x)
#define Semaphore_Wait(x)             WaitForSingleObject(x,INFINITE)
#define Semaphore_Try_Wait(x)         (WaitForSingleObject(x,0)==WAIT_OBJECT_0)
#define Semaphore_Post(x)             ReleaseSemaphore(x,1,NULL)
#define UNIX_PTR                      unsigned __stdcall 
#define Get_LocalTime(x,y)            (localtime_s(y, x), y)
#define Get_NextToken(x,y,z)          strtok(x,y)

#define snprintf              _snprintf
#define vsnprintf             _vsnprintf
#define unlink                _unlink

#endif

// now we force to carry the flags
//#if defined (_DEBUG) || defined (DEBUG)
#define SET_FLAG(x,y) x=(x<<4)+y
//#else
//#define SET_FLAG(x,y)
//#endif

int Do_Sleep   (long x);
int Do_mSleep  (unsigned int x);

#define ensure_free(x)  if(x) { free (x); x = NULL; } 

#ifdef DAPI
#define EXTRA_MESSAGE_DATA 20
#else
#define EXTRA_MESSAGE_DATA 4
#endif

#ifdef DLOG
namespace _DLOG_ {
#endif


struct Mss_Rec
{
  char        * buf;
  int           status;
  int           len;
  const char  * source;
  char  extra_data[EXTRA_MESSAGE_DATA];
};

#define MAX_ARRSTACK_SIZE 10
// this class handles the vector of objects of class T
template <class T>
class CvArray
{
  int max_used;
  int m_for_number;
  int m_pstack;
  int m_lstack[MAX_ARRSTACK_SIZE];

public:
  int max_number;
  T *T_ptr;

  CvArray (int max=20);
 ~CvArray (void);
  
  inline T &operator [](int item_num) {return T_ptr[item_num];}
  inline T &operator ()(int item_num) {if (item_num >= max_number)
                                         Set_New_Limit (item_num*2);
                                       return T_ptr[item_num];}
  void operator = (CvArray <T> &X);
  void Insert (T val);
  void Erase  (T val);
  int  Find   (T val);
  void Clean  (void);
  void End    (void);
  void Changed(void);
  T   *Begin  (void);
  T   *Next   (void);
  void SetZero(void);
#ifdef USE_TRANSACTIONS
  char *Init_Transaction   (char *buff);
  char *Cancel_Trasaction  (char *buff);
  char *Commit_Transaction (char *buff);
#endif
  void Set_New_Limit (int new_max_number);
};


class CMss_Arr
{
  Mutex_Type      m_lock;
  Mutex_Type      m_lock_collect;
  Mutex_Type      m_lock_withdraw;
  Semaphore_Type  m_sem;
  int m_num_recs;
  int m_next_withdraw;
  int m_next_collect;
  int m_rec_len;
  int m_type;
  int m_transaction;
  int m_transaction_pending;
  CvArray <int>  m_transaction_array;
  unsigned int m_collected; // also posted semaphores
  unsigned int m_withdrawn;
  unsigned int m_pending;
  unsigned int m_dropped;
  unsigned int m_data_packets;
  struct Mss_Rec *m_records;

public:
  unsigned int m_flag;
  // insert a new packet
  void Collect      (int status, const char *buf, int len, const char *source = NULL, const char *extra_data = NULL);
  void Init_Collect (int status, char ** buf);
  void Take_Collect (int len);
  void Dropped       (void);

  // withdraw a packet
  // return the status of the packet, -1 means no record available
  int  Withdraw (char *buf, int *len, const char **source = NULL, char *extra_data = NULL);
  // init withdrawing to take the buffer directly
  int  Init_Withdraw (char **buf, int *len, const char **source = NULL, char *extra_data = NULL);
  // concludes the inited withdrawing
  void Take_Withdraw (void);

  void Initialize (void);

  // is any packet available?
  // inline ???
  int Check_For_Any_Pkt_Without_Wait (void);
  int Wait_For_Any_Pkt (void);
  int Is_Any_Pkt (void);

#ifdef USE_TRANSACTIONS
  // init transaction
  void Init_Transaction   (char *buff);
  // cancel transaction
  void Cancel_Transaction (char *buff);
  // commit transaction
  void Commit_Transaction (char *buff);
#endif

  // take a packet
  // return last setted type
  int Change_Type (int new_type);

  void Resize (int new_num_recs);

  unsigned int Get_Total_Collected  (void) {return m_collected ;};
  unsigned int Get_Total_Withdrawn  (void) {return m_withdrawn;};
  unsigned int Get_Total_Pending    (void) {return m_pending;};
  unsigned int Get_Total_Dropped    (void) {return m_dropped;};
  unsigned int Get_Total_DataPacket (void) {return m_data_packets;};
  int           Get_Total_Num_Records(void) {return m_num_recs;};
  int           Get_Rec_Len          (void) {return m_rec_len;};

  CMss_Arr (int init_num_records = CMSS_INITIAL_NUM_RECORDS, 
            int rec_len =CMSS_RECORD_LEN,  
            int type = CMSS_FIFO);
  ~CMss_Arr ();

};

////////////
// CvArray //
////////////

template <class T>
CvArray<T>::  CvArray (int max)
{
  T_ptr      = NULL;
  max_number = 0;
  max_used   = 0;
  m_pstack   = -1;
  Set_New_Limit (max);
}

template <class T>
CvArray<T>:: ~CvArray (void)
{
  free (T_ptr);
}
  
template <class T>
void CvArray<T>::Set_New_Limit (int new_max_number)
{ _SA_(0) 

  // check if the new limit is under the maximum value
  if (new_max_number <= max_number)
    return;

  T_ptr = (T *) realloc (T_ptr, new_max_number * sizeof(T));
  memset (&T_ptr[max_number], 0, (size_t)(new_max_number-max_number)*sizeof(T));

  max_number = new_max_number;
}

template <class T>
void CvArray<T>::operator = (CvArray <T> &X)
{ _SA_(0) 
  int max;
  if (max_number < X.max_number)
  {
    Set_New_Limit (X.max_number);
    max = max_number;
  }
  else
    max = X.max_number;

  memcpy (T_ptr, X.T_ptr, max * sizeof(T));
}

template <class T>
void CvArray<T>::Insert (T val)
{ _SA_(0) 
  m_for_number = -1;
  if (max_used + 1 >= max_number) 
    Set_New_Limit ((max_used+1) * 2);

  T_ptr [max_used++] = val;
}

template <class T>
void CvArray<T>::Erase  (T val)
{ _SA_(0) 
  int i;

  Changed ();
  for (i=0; i<max_used; ++i)
    if (T_ptr [i] == val)
    {
      memcpy (&T_ptr [i], &T_ptr [i+1], (max_used - i - 1)*sizeof (T));
      --max_used;
      return;
    }
}

template <class T>
int CvArray<T>::Find   (T val)
{ _SA_(0) 
  int i;

  for (i=0; i<max_used; ++i)
    if (T_ptr [i] == val)
      return i;
  return -1;
}

template <class T>
void CvArray<T>::SetZero  (void)
{ _SA_(0) 
  if (!T_ptr)
    return;
  memset (T_ptr, 0, sizeof (T) * max_number);
}

template <class T>
void CvArray<T>::Clean  (void)
{ _SA_(0) 
  Changed ();
  max_used      = 0;
}

template <class T>
T   *CvArray<T>::Begin  (void)
{ _SA_(0) 
  extern int Glb_Max_Arr_Stack_Size;
  if (m_pstack >= 0)
    m_lstack [m_pstack] = m_for_number;

  ++m_pstack;
  if (m_pstack > Glb_Max_Arr_Stack_Size)
    Glb_Max_Arr_Stack_Size = m_pstack;

  if(m_pstack >= MAX_ARRSTACK_SIZE)
    --m_pstack;

  m_for_number = 0;
  if (m_for_number < max_used)
    return &T_ptr [m_for_number];

  End ();

  return NULL;
}

template <class T>
T   *CvArray<T>::Next   (void)
{ _SA_(0) 
  ++m_for_number;
  if (m_for_number < max_used)
    return &T_ptr [m_for_number];

  End ();

  return NULL;
}

template <class T>
void CvArray<T>::End (void)
{ _SA_(0) 
  --m_pstack;
  if (m_pstack >= 0 && m_for_number >= 0)
    m_for_number = m_lstack [m_pstack];

  if (m_pstack < -1)
    m_pstack = -1;
}

template <class T>
void CvArray<T>::Changed (void)
{ _SA_(0) 
  m_for_number  = -1;
  if (m_pstack > 0)
  {
    int i;
    for (i=0; i< m_pstack; ++i)
      m_lstack [m_pstack] = m_for_number;
  }
}

#ifdef DMEAPP

extern char *TRANS_save_data     (char *buff_ptr, void *source, int length);
extern char *TRANS_restore_data  (char *buff_ptr, void *source, int length);

template <class T>
char *CvArray<T>::Init_Transaction   (char *buff)
{ _SA_(0) 
  buff = TRANS_save_data (buff, &max_used,      sizeof (max_used));
  buff = TRANS_save_data (buff, T_ptr,          sizeof (T)*max_used);
  buff = TRANS_save_data (buff, &m_lstack,      sizeof (m_lstack));
  buff = TRANS_save_data (buff, &m_pstack,      sizeof (m_pstack));
  buff = TRANS_save_data (buff, &m_for_number,  sizeof (m_for_number));
  return buff;
}

template <class T>
char *CvArray<T>::Cancel_Trasaction  (char *buff)
{ _SA_(0) 
  buff = TRANS_restore_data (buff, &max_used,      sizeof (max_used));
  buff = TRANS_restore_data (buff, T_ptr,         sizeof (T)*max_used);
  buff = TRANS_restore_data (buff, &m_lstack,      sizeof (m_lstack));
  buff = TRANS_restore_data (buff, &m_pstack,      sizeof (m_pstack));
  buff = TRANS_restore_data (buff, &m_for_number,  sizeof (m_for_number));
  return buff;
}

template <class T>
char *CvArray<T>::Commit_Transaction (char *buff)
{ _SA_(0) 
  int used;
  buff = TRANS_restore_data (buff, &used, sizeof (max_used));
  buff += sizeof (T) * used;
  buff += sizeof (m_lstack);
  buff += sizeof (m_pstack);
  buff += sizeof (m_for_number);
  return buff;
}

#endif // DMEAPP

#undef GLOBAL

#ifdef DLOG
}
using namespace _DLOG_;
#endif

#endif // _ARRM_H_
//////////////////////////////////////////////////////////////////////////////
