//////////////////////////////////////////////////////////////////////////////
//  DCON.H   : Functions to control DME comunications              
//////////////////////////////////////////////////////////////////////////////

#ifndef DCON_INCLUDE_DEFINED
#define DCON_INCLUDE_DEFINED

#define DCON_MAX_MSG_LEN  5000

#define DCON_SLEEP_TIME       1
#define DCON_MILISLEEP_TIME   100

#define DME_DEFAULT_CONNECTION_RETRY_WAIT       10              // seconds
#define DEFAULT_CONNECTION_RETRY_WAIT           DCON_SLEEP_TIME // seconds. DGI uses original retry wait time.

#define DCON_ST_OPENING   1
#define DCON_ST_OPENCHK   2
#define DCON_ST_NORMCON   4
#define DCON_ST_CLOSE     5
#define DCON_ST_TERMIN    6
#define DCON_ST_CLOSE_AND_RESTART  7

#define DCON_TICKING_SPAN_TIME  2

#define DCON_NEXT_STATE_VOID         0
#define DCON_NEXT_STATE_TERMINATE    1
#define DCON_NEXT_STATE_RESTART      2

#define DCON_MAX_DS_SERVERS          36

extern const char *DCON_Status[];

struct DCON_DESCR
{
          unsigned int   number_of_hosts;
          unsigned int   current_host;
          unsigned int   fhost    [DCON_MAX_DS_SERVERS];
          unsigned short tcp_port [DCON_MAX_DS_SERVERS];
          int            modesend;
          int            use_ssl;
          int            ticking_message;
          int            expire_time;
          char           time_conn_disconn_str [20];

          CMss_Arr      *pRecv_Mss;
          CMss_Arr      *pSend_Mss;
          CMss_Arr      *pLog_Mss;
          const char    *id_ptr;
          int            id_len;
          const char    *thread_name_i;    // process thread name used to identify thread alert functions
          unsigned       thread_prg_id_i;  // process thread id multiple of 2:  unsigned, one bit set
          const char    *thread_name_s;    // do_send thread name used to identify thread alert functions
          unsigned       thread_prg_id_s;  // do_send thread id multiple of 2:  unsigned, one bit set

volatile  int            DCON_state;
          Mutex_Type     DCON_lock;
          int            DCON_pauses;
          int            DCON_mpauses;
volatile  int            DCON_terminate;
          unsigned int  send_flag;
          int           *sock_state;
          WSocket        DWsock;
          // etc.etc
};

UNIX_PTR DCON_Do_Send       (void *ptr);
UNIX_PTR DCON_Init          (void *Con_DESCR);

void DCON_Compression_Test (struct DCON_DESCR *pCon_DESCR, char *buff, unsigned short len);

void DCON_Do_Not_Receive_Msgs (void);

int  DCON_Skip_Header      (const char **mssg, int *mssg_len, const char *source);
void DCON_Put_Message_In_Recv_Array (CMss_Arr      *recv_mss,
                                     int            status, 
                                     const char    *mssg, 
                                     int            mssg_len,
                                     const char    *source,
                                     char          *rle_mssg_compressed     = NULL, // By default messages to recv array are not compressed.
                                     int            rle_mssg_compressed_len = 0,
                                     int            skip_header=0);

void GetCompilationPlatform(char* pltfrm);

#ifdef USE_DCON_MON_MESS
// wdf-ds  header for communication
#if !(USE_SOLARIS || USE_LINUX)
#pragma pack (push,2)
#endif

#define  MAX_MON_BUF_LEN   4000

struct MON_MESS_DESTR {
    unsigned short flags;        /* Banderas para indicar procedencia, destino, etc */
    unsigned short mess_long;    /* Longitud total del mensaje */
    unsigned short mess_type;    /* Tipo de mensaje,
                                      coneccion,
                                      aceptacion_de_coneccion,
                                      negacion de coneccion,
                                      despido,
                                      envio_mensaje,
                                      consultas
                                      echo, etc */
    unsigned short priority;    /* Prioridad del mensaje */
    unsigned int eq_src_name;  /* Codigo nombre del equipo que envia */
    char     mess_buff [MAX_MON_BUF_LEN];
          /* Contenido de mensajes entre dfs */
          /* unsigned num_dests;                     numero de destinatarios          */
          /* char     destinatarios [num_dests * sizeof (long)]                       */
          /* unsigned df_mess_len;                   Incluye longitud de df_mess_type */

          /* char     df_mess_type[3];                                                */
          /* char     mensaje[dfmess_len - 3]                                         */
    };

#define  ENVIA_EQUIPO            1
#define  MENSAJE_NORMAL        100  // indica idx de tablas de 32 bits.
#define  HEAD_FIX_DATA_LEN      16
#define  HEAD_FIX_LEN           12
#define  HEAD_TO_PRIORITY_LEN    6

class CDestinationsList;

unsigned short DCON_Set_Header  (struct MON_MESS_DESTR  *dest_area,
                                 const char             *packet,
                                 int                     packet_len,
                                 CDestinationsList      *c_dest,
                                 int                     priority);

#if !(USE_SOLARIS || USE_LINUX)
#pragma pack (pop)
#endif

#endif // USE_DCON_MON_MESS

#endif
//////////////////////////////////////////////////////////////////////////////
