//////////////////////////////////////////////////////////////////////////////
// field.h
//
// CField definition.
//////////////////////////////////////////////////////////////////////////////

#ifndef __FIELD_H__
#define __FIELD_H__

#include <time.h>
#include "c_calc.h"

#define FIRST_CHAR_NUMBER_CODE      0x30

#define CLEAR(x) memset(x, 0, sizeof(x))

// This is the char used to link fields
#define FIELD_JOIN_CHAR         '.'
#define FIELD_START_ARRAY_CHAR  '['
#define FIELD_END_ARRAY_CHAR    ']'

class CField;
typedef void * (CField_Ref_Fun)(CField *this_ptr, int array_index, int *reference_fail, int report_referenced_error);
void FIELD_Report_No_Definition (int id, int msg);

struct S_Bits
{
  unsigned int grp_bits;
  unsigned int position;
  unsigned int set_value;
};

typedef char *policy_f(CField *);

class CField
{
public:
    int       m_field_id;
    CMessage *m_parent;
    char     *m_data;
    char     *m_msg_field_init_pointer;
    int       m_type;
    int       m_len;
    int       m_decimal_len;
    int       m_displacement;
    int       m_no_change_message_flag; // signal that if this field is changed the message is not considered changed
#ifdef DAPI
    int       m_index;    // usado como ndice en campos arreglo (por ahora.. ndice simple).
    int       m_elem_len; // usado como ndice en campos arreglo
    policy_f *m_policy_f;
#endif
    CField (CMessage *msg_parent, int field_id, int p_pos);
    virtual ~CField();

#ifdef DAPI
#define MAX_REF_CALL  30

static CField_Ref_Fun  *m_ref_func    [MAX_REF_CALL];
    int           m_ref_value;
    const char  * m_field_name;
    int           m_field_name_code;
    void  Set_Name          (const char *field_name, int field_name_code);
    char *Do_Matching_Name  (char *name, int name_code);
    bool  Is_Named          (void);
    void *Do_Reference_Call (int array_index=0, int report_reference_error=0, int *reference_fail = NULL);
static void Set_Reference_Call (int ref_value, CField_Ref_Fun *func_ptr);
#endif
    bool      IsEmptyField(void);
    bool      IsOnlyChar  (int c_val, int len=0);
    void      Clean       (void);
    int       MemCmp      (void *comp);
    inline char    *Get_Data_Ptr (void) 
    {
#ifdef DAPI
      if (m_policy_f)
        return (*m_policy_f)(this);
#endif
      return m_data = (m_parent->m_data + m_displacement);
    }
    virtual   char *operator >>(void *dest);
    virtual   char *GetStr (void *dest, int offset, int maxlen);
    virtual   void  SetStr (const char*source);
    void          Set_Bits (S_Bits *s_bits);
    void          Set_Bits (unsigned int grp_bits, unsigned int position, unsigned int value);
    unsigned int  Get_Bits (S_Bits *s_bits);
    unsigned int  Get_Bits (unsigned int grp_bits, unsigned int position);
    void          Init_Bits(S_Bits *s_bits);
    void          Init_Bits(unsigned int grp_bits, unsigned int value);

    virtual operator int()           = 0;
    virtual operator unsigned int() = 0;
    inline     operator const char ()   
    {
      return *(m_parent->m_data + m_displacement);
    }
    inline     operator char* ()        
    {
#ifdef DAPI
      if (m_policy_f)
        return (*m_policy_f)(this);
      return m_parent->m_data + m_displacement + (m_index > 0 ? (m_index-1)*m_elem_len : 0); 
#else
      return m_parent->m_data + m_displacement;
#endif
    }
    inline     operator const void *()  
    {
#ifdef DAPI
      if (m_policy_f)
        return (*m_policy_f)(this);
      return m_parent->m_data + m_displacement + (m_index > 0 ? (m_index-1)*m_elem_len : 0); 
#else
      return m_parent->m_data + m_displacement;
#endif
    }
    inline const char *Get_Ptr   (void) 
    {
#ifdef DAPI
      if (m_policy_f)
        return (*m_policy_f)(this);
      return m_parent->m_data + m_displacement + (m_index > 0 ? (m_index-1)*m_elem_len : 0); 
#else
      return m_parent->m_data + m_displacement;
#endif
    }
    virtual operator CALC_MNUM *() = 0;

    virtual bool operator==(CField &X) = 0;
    virtual bool operator!=(CField &X) = 0;
    virtual bool operator <(CField &X) = 0;
    virtual bool operator >(CField &X) = 0;
    virtual int  operator =(CField &X) = 0;
    virtual int  operator +(CField &X) = 0;
    virtual int  operator -(CField &X) = 0;
    virtual CField &operator += (int value);

    //CNumAsciiInt
    virtual bool operator==(const int val) = 0;
    virtual bool operator!=(const int val) = 0;
    virtual int  operator =(const int val) = 0;
    //CAlpha
    virtual bool operator==(const char *val) = 0;
    virtual bool operator!=(const char *val) = 0;
    virtual int  operator =(const char *val) = 0;
    //CShort
    virtual bool operator==(const short val) = 0;
    virtual bool operator!=(const short val) = 0;
    virtual int  operator =(const short val) = 0;
    //CUnsignedShort
    virtual bool operator==(const unsigned short val) = 0;
    virtual bool operator!=(const unsigned short val) = 0;
    virtual int  operator =(const unsigned short val) = 0;
    //CUnsignedInt
    virtual bool operator==(const unsigned int val) = 0;
    virtual bool operator!=(const unsigned int val) = 0;
    virtual int  operator =(const unsigned int val) = 0;
    // CNumAsciiDouble
    virtual bool   operator==(CALC_MNUM * val) = 0;
    virtual bool   operator!=(CALC_MNUM * val) = 0;
    virtual bool   operator <(CALC_MNUM * val) = 0;
    virtual bool   operator >(CALC_MNUM * val) = 0;
    virtual int    operator =(CALC_MNUM * val) = 0;
    virtual CALC_MNUM * operator +(CALC_MNUM * val) = 0;
    virtual CALC_MNUM * operator -(CALC_MNUM * val) = 0;
};

class CNumAsciiInt : public CField
{
public:
    int m_value;
    CALC_MNUM m_svalue[COMMON_CALC_SIZE];
    char   m_str_value[16];

    CNumAsciiInt(CMessage *msg_parent, int field_id, int p_pos);
    ~CNumAsciiInt() {};

       void Set_Value(void);
inline int  Get_Value(void)
       {
         if (!*m_msg_field_init_pointer)
           Set_Value ();
         return m_value;
       }

    operator int();
    operator unsigned int();
    operator CALC_MNUM *();

    bool operator==(CField &X);
    bool operator!=(CField &X); 
    bool operator <(CField &X); 
    bool operator >(CField &X); 
    int  operator =(CField &X); 
    int  operator +(CField &X); 
    int  operator -(CField &X); 

    bool operator==(CNumAsciiInt &X);
    bool operator!=(CNumAsciiInt &X);
    bool operator <(CNumAsciiInt &X);
    bool operator >(CNumAsciiInt &X);
    int  operator =(CNumAsciiInt &X);
    int  operator +(CNumAsciiInt &X);
    int  operator -(CNumAsciiInt &X);
    //CNumAsciiInt
    bool operator==(const int val);
    bool operator!=(const int val);
    int  operator =(const int val);
    //CAlpha
    bool operator==(const char *val); 
    bool operator!=(const char *val); 
    int  operator =(const char *val); 
    //CShort
    bool operator==(const short val); 
    bool operator!=(const short val); 
    int  operator =(const short val); 
    //CUnsignedShort
    bool operator==(const unsigned short val); 
    bool operator!=(const unsigned short val); 
    int  operator =(const unsigned short val); 
    //CUnsignedInt
    virtual bool operator==(const unsigned int val);
    virtual bool operator!=(const unsigned int val);
    virtual int  operator =(const unsigned int val);
    // CNumAsciiDouble
    bool   operator==(CALC_MNUM *val); 
    bool   operator!=(CALC_MNUM *val); 
    bool   operator <(CALC_MNUM *val); 
    bool   operator >(CALC_MNUM *val); 
    int    operator =(CALC_MNUM *val); 
    CALC_MNUM *operator +(CALC_MNUM *val);
    CALC_MNUM *operator -(CALC_MNUM *val);

    char *operator >>(void *dest);
};

class CAlpha: public CField
{
public:
    char *m_value;
    int   m_alpha_type;

    CAlpha(CMessage *msg_parent, int field_id, int p_pos);
    ~CAlpha() {};

       void  Set_Value(void);
inline char *Get_Value(void)
       {
         if (!*m_msg_field_init_pointer)
           Set_Value ();
         return m_value;
       }

    operator int();
    operator unsigned int();
    operator CALC_MNUM *();

    bool operator==(CField &X);
    bool operator!=(CField &X);
    bool operator <(CField &X);
    bool operator >(CField &X);
    int  operator =(CField &X);
    int  operator +(CField &X);
    int  operator -(CField &X);

    bool operator==(CAlpha &X);
    bool operator!=(CAlpha &X);
    bool operator <(CAlpha &X);
    bool operator >(CAlpha &X);
    int  operator =(CAlpha &X);
    int  operator +(CAlpha &X);
    int  operator -(CAlpha &X);
    //CNumAsciiInt
    bool operator==(const int val);
    bool operator!=(const int val);
    int  operator =(const int val);
    //CAlpha
    bool operator==(const char *val);
    bool operator!=(const char *val);
    int  operator =(const char *val);
    //CShort
    bool operator==(const short val);
    bool operator!=(const short val);
    int  operator =(const short val);
    //CUnsignedShort
    bool operator==(const unsigned short val);
    bool operator!=(const unsigned short val);
    int  operator =(const unsigned short val);
    //CUnsignedInt
    virtual bool operator==(const unsigned int val);
    virtual bool operator!=(const unsigned int val);
    virtual int  operator =(const unsigned int val);
    // CNumAsciiDouble
    bool   operator==(CALC_MNUM *val);
    bool   operator!=(CALC_MNUM *val);
    bool   operator <(CALC_MNUM *val);
    bool   operator >(CALC_MNUM *val);
    int    operator =(CALC_MNUM *val);
    CALC_MNUM * operator +(CALC_MNUM *val);
    CALC_MNUM * operator -(CALC_MNUM *val);
    char *operator >>(void *dest);
    char *GetStr (void *dest, int offset, int maxlen);
    void  SetStr (const char* source);
};

class CShort: public CField
{
public:
    short m_value;
    CALC_MNUM m_svalue[COMMON_CALC_SIZE];
    char   m_str_value[8];

    CShort(CMessage *msg_parent, int field_id, int p_pos);
    ~CShort() {};

    void  Set_Value (void);
inline short Get_Value(void)
       {
         if (!*m_msg_field_init_pointer)
           Set_Value ();
         return m_value;
       }

    operator int();
    operator unsigned int();
    operator CALC_MNUM *();

    bool operator==(CField &X); 
    bool operator!=(CField &X);
    bool operator <(CField &X); 
    bool operator >(CField &X); 
    int  operator =(CField &X); 
    int  operator +(CField &X); 
    int  operator -(CField &X); 

    bool operator==(CShort &X);
    bool operator!=(CShort &X);
    bool operator <(CShort &X);
    bool operator >(CShort &X);
    int  operator =(CShort &X);
    short operator+(CShort &X);
    short operator-(CShort &X);
    //CNumAsciiInt
    bool operator==(const int val);
    bool operator!=(const int val);
    int  operator =(const int val);
    //CAlpha
    bool operator==(const char *val); 
    bool operator!=(const char *val);
    int  operator =(const char *val);
    //CShort
    bool operator==(const short val);
    bool operator!=(const short val);
    int  operator =(const short val);
    //CUnsignedShort
    bool operator==(const unsigned short val);
    bool operator!=(const unsigned short val);
    int  operator =(const unsigned short val);
    //CUnsignedInt
    virtual bool operator==(const unsigned int val);
    virtual bool operator!=(const unsigned int val);
    virtual int  operator =(const unsigned int val);
    //CNumAsciiDouble
    bool   operator==(CALC_MNUM *val);
    bool   operator!=(CALC_MNUM *val);
    bool   operator <(CALC_MNUM *val);
    bool   operator >(CALC_MNUM *val);
    int    operator =(CALC_MNUM *val);
    CALC_MNUM * operator +(CALC_MNUM *val);
    CALC_MNUM * operator -(CALC_MNUM *val);
};

class CUnsignedShort: public CField
{
public:
    unsigned short m_value;
    CALC_MNUM m_svalue[COMMON_CALC_SIZE];
    char   m_str_value[8];

    CUnsignedShort(CMessage *msg_parent, int field_id, int p_pos);
    ~CUnsignedShort() {};

       void           Set_Value (void);
inline unsigned short Get_Value(void)
       {
         if (!*m_msg_field_init_pointer)
           Set_Value ();
         return m_value;
       }

    operator int();
    operator unsigned int();
    operator CALC_MNUM *();

    bool operator==(CField &X); 
    bool operator!=(CField &X);
    bool operator <(CField &X); 
    bool operator >(CField &X); 
    int  operator =(CField &X); 
    int  operator +(CField &X); 
    int  operator -(CField &X); 

    bool operator==(CUnsignedShort &X);
    bool operator!=(CUnsignedShort &X);
    bool operator <(CUnsignedShort &X);
    bool operator >(CUnsignedShort &X);
    int  operator =(CUnsignedShort &X);
    unsigned short operator +(CUnsignedShort &X);
    short operator -(CUnsignedShort &X);
    //CNumAsciiInt
    bool operator==(const int val);
    bool operator!=(const int val);
    int  operator =(const int val);
    //CAlpha
    bool operator==(const char *val); 
    bool operator!=(const char *val);
    int  operator =(const char *val);
    //CShort
    bool operator==(const short val);
    bool operator!=(const short val);
    int  operator =(const short val);
    //CUnsignedShort
    bool operator==(const unsigned short val);
    bool operator!=(const unsigned short val);
    int  operator =(const unsigned short val);
    //CUnsignedInt
    virtual bool operator==(const unsigned int val); 
    virtual bool operator!=(const unsigned int val); 
    virtual int  operator =(const unsigned int val); 
    // CNumAsciiDouble
    bool   operator==(CALC_MNUM *val); 
    bool   operator!=(CALC_MNUM *val); 
    bool   operator <(CALC_MNUM *val); 
    bool   operator >(CALC_MNUM *val); 
    int    operator =(CALC_MNUM *val); 
    CALC_MNUM * operator +(CALC_MNUM *val); 
    CALC_MNUM * operator -(CALC_MNUM *val); 
};

class CInteger: public CField
{
public:
    int m_value;
    CALC_MNUM m_svalue[COMMON_CALC_SIZE];
    char   m_str_value[12];

    CInteger(CMessage *msg_parent, int field_id, int p_pos);
    ~CInteger() {};

    void  Set_Value (void);
inline short Get_Value(void)
       {
         if (!*m_msg_field_init_pointer)
           Set_Value ();
         return m_value;
       }

    operator int();
    operator unsigned int();
    operator CALC_MNUM *();

    bool operator==(CField &X); 
    bool operator!=(CField &X);
    bool operator <(CField &X); 
    bool operator >(CField &X); 
    int  operator =(CField &X); 
    int  operator +(CField &X); 
    int  operator -(CField &X); 

    bool operator==(CInteger &X);
    bool operator!=(CInteger &X);
    bool operator <(CInteger &X);
    bool operator >(CInteger &X);
    int  operator =(CInteger &X);
    short operator+(CInteger &X);
    short operator-(CInteger &X);
    //CNumAsciiInt
    bool operator==(const int val);
    bool operator!=(const int val);
    int  operator =(const int val);
    //CAlpha
    bool operator==(const char *val); 
    bool operator!=(const char *val);
    int  operator =(const char *val);
    //CShort
    bool operator==(const short val);
    bool operator!=(const short val);
    int  operator =(const short val);
    //CUnsignedShort
    bool operator==(const unsigned short val);
    bool operator!=(const unsigned short val);
    int  operator =(const unsigned short val);
    //CUnsignedInt
    virtual bool operator==(const unsigned int val);
    virtual bool operator!=(const unsigned int val);
    virtual int  operator =(const unsigned int val);
    //CNumAsciiDouble
    bool   operator==(CALC_MNUM *val);
    bool   operator!=(CALC_MNUM *val);
    bool   operator <(CALC_MNUM *val);
    bool   operator >(CALC_MNUM *val);
    int    operator =(CALC_MNUM *val);
    CALC_MNUM * operator +(CALC_MNUM *val);
    CALC_MNUM * operator -(CALC_MNUM *val);
};

class CUnsignedInt: public CField
{
public:
    unsigned int m_value;
    CALC_MNUM m_svalue[COMMON_CALC_SIZE];
    char   m_str_value[12];

    CUnsignedInt(CMessage *msg_parent, int field_id, int p_pos);
    ~CUnsignedInt() {};

       void           Set_Value (void);
inline unsigned int  Get_Value(void)
       {
         if (!*m_msg_field_init_pointer)
           Set_Value ();
         return m_value;
       }

    operator int();
    operator unsigned int();
    operator CALC_MNUM *();

    bool operator==(CField &X);
    bool operator!=(CField &X);
    bool operator <(CField &X);
    bool operator >(CField &X);
    int  operator =(CField &X);
    int  operator +(CField &X);
    int  operator -(CField &X);

    bool operator==(CUnsignedInt &X);
    bool operator!=(CUnsignedInt &X);
    bool operator <(CUnsignedInt &X);
    bool operator >(CUnsignedInt &X);
    int  operator =(CUnsignedInt &X);
    unsigned int operator +(CUnsignedInt &X);
    int operator -(CUnsignedInt &X);
    //CNumAsciiInt
    bool operator==(const int val);
    bool operator!=(const int val);
    int  operator =(const int val);
    //CAlpha
    bool operator==(const char *val); 
    bool operator!=(const char *val); 
    int  operator =(const char *val); 
    //CShort
    bool operator==(const short val); 
    bool operator!=(const short val); 
    int  operator =(const short val); 
    //CUnsignedShort
    bool operator==(const unsigned short val);
    bool operator!=(const unsigned short val);
    int  operator =(const unsigned short val);
    //CUnsignedInt
    virtual bool operator==(const unsigned int val);
    virtual bool operator!=(const unsigned int val);
    virtual int  operator =(const unsigned int val);
    // CNumAsciiDouble
    bool   operator==(CALC_MNUM *val); 
    bool   operator!=(CALC_MNUM *val); 
    bool   operator <(CALC_MNUM *val); 
    bool   operator >(CALC_MNUM *val); 
    int    operator =(CALC_MNUM *val); 
    CALC_MNUM * operator +(CALC_MNUM *val); 
    CALC_MNUM * operator -(CALC_MNUM *val); 
};

class CNumAsciiDouble: public CField
{
public:
    char   m_str_value[28];
    CALC_MNUM m_value[COMMON_CALC_SIZE];

    CNumAsciiDouble(CMessage *msg_parent, int field_id, int p_pos);
    ~CNumAsciiDouble() {};

       void   Set_Value (void);
inline CALC_MNUM *Get_Value(void)
       {
         if (!*m_msg_field_init_pointer)
           Set_Value ();
         return m_value;
       }

    operator int();
    operator unsigned int();
    operator CALC_MNUM *();

    bool operator==(CField &X); 
    bool operator!=(CField &X); 
    bool operator <(CField &X); 
    bool operator >(CField &X); 
    int  operator =(CField &X); 
    int  operator +(CField &X); 
    int  operator -(CField &X); 

    bool operator==(CNumAsciiDouble &X);
    bool operator!=(CNumAsciiDouble &X);
    bool operator <(CNumAsciiDouble &X);
    bool operator >(CNumAsciiDouble &X);
    int  operator =(CNumAsciiDouble &X);
    CALC_MNUM * operator +(CNumAsciiDouble &X);
    CALC_MNUM * operator -(CNumAsciiDouble &X);
    //CNumAsciiInt
    bool operator==(const int val);
    bool operator!=(const int val);
    int  operator =(const int val);
    //CAlpha
    bool operator==(const char *val); 
    bool operator!=(const char *val); 
    int  operator =(const char *val);
    //CShort
    bool operator==(const short val); 
    bool operator!=(const short val); 
    int  operator =(const short val); 
    //CUnsignedShort
    bool operator==(const unsigned short val);
    bool operator!=(const unsigned short val);
    int  operator =(const unsigned short val);
    //CUnsignedInt
    virtual bool operator==(const unsigned int val);
    virtual bool operator!=(const unsigned int val);
    virtual int  operator =(const unsigned int val);
    // CNumAsciiDouble
    bool   operator==(CALC_MNUM *val);
    bool   operator!=(CALC_MNUM *val);
    bool   operator <(CALC_MNUM *val);
    bool   operator >(CALC_MNUM *val);
    int    operator =(CALC_MNUM *val);
    CALC_MNUM * operator +(CALC_MNUM *val);
    CALC_MNUM * operator -(CALC_MNUM *val);

    char *operator >>(void *dest);
};

class CDate: public CField
{
public:
    char *m_value;
    struct tm m_tm;
    int       m_tm_valid;
    char      m_str_value[16];

    CDate(CMessage *msg_parent, int field_id, int p_pos);
    ~CDate() {};

       void  Set_Value(void);
inline char *Get_Value(void)
       {
         if (!*m_msg_field_init_pointer)
           Set_Value ();
         return m_value;
       }

    operator int();
    operator unsigned int();
    operator CALC_MNUM *();

    bool operator==(CField &X);
    bool operator!=(CField &X);
    bool operator <(CField &X);
    bool operator >(CField &X);
    int  operator =(CField &X);
    int  operator +(CField &X);
    int  operator -(CField &X);

    bool operator==(CDate &X);
    bool operator!=(CDate &X);
    bool operator <(CDate &X);
    bool operator >(CDate &X);
    int  operator =(CDate &X);
    int  operator +(CDate &X);
    int  operator -(CDate &X);
    //CNumAsciiInt
    bool operator==(const int val);
    bool operator!=(const int val);
    int  operator =(const int val);
    //CAlpha
    bool operator==(const char *val);
    bool operator!=(const char *val);
    int  operator =(const char *val);
    //CShort
    bool operator==(const short val);
    bool operator!=(const short val);
    int  operator =(const short val);
    //CUnsignedShort
    bool operator==(const unsigned short val);
    bool operator!=(const unsigned short val);
    int  operator =(const unsigned short val);
    //CUnsignedInt
    virtual bool operator==(const unsigned int val);
    virtual bool operator!=(const unsigned int val);
    virtual int  operator =(const unsigned int val);
    // CNumAsciiDouble
    bool   operator==(CALC_MNUM *val);
    bool   operator!=(CALC_MNUM *val);
    bool   operator <(CALC_MNUM *val);
    bool   operator >(CALC_MNUM *val);
    int    operator =(CALC_MNUM *val);
    CALC_MNUM * operator +(CALC_MNUM *val);
    CALC_MNUM * operator -(CALC_MNUM *val);
    char *operator >>(void *dest);
    char *StrCpy (void *dest, int offset, int maxlen);
    char *GetStr (void *dest, int offset, int maxlen);
    CField &operator += (int value);
};

void                  FIELD_Set_Bits                    (unsigned char *ptr, 
                                                          unsigned int grp_bits, 
                                                          unsigned int position, 
                                                          unsigned int value);
unsigned int          FIELD_Get_Bits                    (unsigned char *ptr, 
                                                          unsigned int grp_bits, 
                                                          unsigned int position);

void                  FIELD_Init_Bits                   (unsigned char *ptr, 
                                                         unsigned int   ptr_len, 
                                                         unsigned int   grp_bits, 
                                                         unsigned int   value);


#endif //#ifndef __FIELD_H__
