/////////////////////////////////////////////////////////////////////////////
// tblequiv.h
//
// Equivalence defined for table code ids.
/////////////////////////////////////////////////////////////////////////////

#ifndef __TBLEQUIV_H__
#define __TBLEQUIV_H__

#include "dmedefs.h"

#ifdef DAPI
#define FIRST_LEVEL_TABLE         1
#define SECOND_LEVEL_TABLE        2
#endif

#define NOT_THE_LAST_ANSWER       0
#define IS_THE_LAST_ANSWER        1

#define TBLEQUIV_NO_SOURCE        0
#define TBLEQUIV_DS_SOURCE        1
#define TBLEQUIV_DME_SOURCE       2
#define TBLEQUIV_DS_DME_SOURCE    (TBLEQUIV_DS_SOURCE | TBLEQUIV_DME_SOURCE)
#define TBLEQUIV_GDI_SOURCE       4
#define TBLEQUIV_DS_GDI_SOURCE    (TBLEQUIV_DS_SOURCE | TBLEQUIV_GDI_SOURCE)

#define START_NO_PROTOCOL_USED           0
// always send all records to adjust table at starting
#define START_DEFAULT_PROTOCOL           1
// uses crc32, hashing protocol to adjust table at starting
#define START_WITH_CRC32_PROTOCOL        2

#define TBLNUM_DYNAMIC_CODE  0
#define TBLNUM_NOT_USED      0

#define TABLE_NOT_PUBLISHED                         1
#define TABLE_PUBLISHED_TO_DGI                      2
#define TABLE_PUBLISHED_TO_DS                       4
#define TABLE_PUBLISHED_ALL                         (TABLE_PUBLISHED_TO_DGI | TABLE_PUBLISHED_TO_DS)
#define TABLE_USE_OWN_UNIQUE_IDX                    8
#define TABLE_NO_LOG_ERRORS_INSERTION_MODIFICATION  16
#define TABLE_PUBLISHED_TO_DS_AND_FROM_DS           (TABLE_PUBLISHED_TO_DS | TABLE_NOT_PUBLISHED)


// Global table codes that are unique for all markets and that
// corresponds to tables that must be received by DME.
#define TABLENUM_DATAMANAGERLIST      8
#define TABLENUM_HISTORYTRANSDAYMEM   11
#define TABLENUM_CONNUSER             16
#define TABLENUM_BRANCHES             22
#define TABLENUM_BRANCHESRFQS        114
#define TABLENUM_USER                 24
#define TABLENUM_MARKETS              27
#define TABLENUM_CURRENCY             28
#define TABLENUM_INDICATIVESMARK      55
#define TABLENUM_INSTITUTIONS         21
#define TABLENUM_INCREMENTS           57
#define TABLENUM_CCYPAIR              903
#define TABLENUM_CONNDMSERVER         81
#define TABLENUM_HOLIDAY              29
#define TABLENUM_RFQREQUEST           100
#define TABLENUM_SUPERGROUP           30    // for voice markets.
#define TABLENUM_VOICEGROUP           31    // for voice markets.
#define TABLENUM_VOICE_TERM           32    // for voice markets.
#define TABLENUM_HISTORY_IC           73    // Statistics for Icap Columbia  
// Global table codes of tables mantained partially by DME.
// It means that DME must receive the table but can add
// records to the table.
#define TABLENUM_PUINSDME             52
// Global table codes of tables mantained by DME.
#define TABLENUM_INSTIT_NUM           50
#define TABLENUM_BRANCH_NUM           51
#define TABLENUM_SPOT                 4
#define TABLENUM_OVERNIGHT            2
#define TABLENUM_CASHRATE             13
#define TABLENUM_BRIDGE               80
#define TABLENUM_COUNTERBRIDGE     32014
#define TABLENUM_DMECONNECTED         53

// Las siguientes cuatro tablas son mantenidas por el DS
// y no son publicadas.
#define TABLENUM_COBRANZAS            90
#define TABLENUM_COMPART_CUPO         87
#define TABLENUM_CONFIG_CUPO          89
#define TABLENUM_MARKETSPOTPRICES     18
#ifdef DMEAPP
#define TABLENUM_INSTPERM             54
#endif

// Las sigueintes tablas son mantenidas por el DM y publicadas a todos
#define TABLENUM_CUPOSL               85 
#define TABLENUM_FACTRIESGO           88 
#define TABLENUM_LIMINSTIT            64 
#define TABLENUM_PENDCOBRANZAS        93 
#define TABLENUM_CUPOSXPL             79 

#define TABLENUM_CUPOSXDIA            32040 

#define TABLENUM_SUMMARYRF           10006
#define TABLENUM_OFERTAS            10011
#define TABLENUM_DEMANDAS           10012

typedef enum _DS_CONNECTION_STATUS 
              {
                DM_SERVER_REC_RECEIVED, 
                PERSISTENT_SYNCHRONIZED
              }DS_CONNECTION_STATUS;

#ifndef DAPI  

enum
{
  ZERO_CODE_ID                     ,
  // Tables mantained by DME
  MARKET_INSTIT_NUM_ID             ,
  MARKET_BRANCH_NUM_ID             ,
  MARKET_CREDIT_ADMINISTRATION_ID  ,
  MARKET_CUPOS_ID                  ,
  MARKET_NAMECHECK_TERM_ID         ,
  BRIDGE_CODE_ID                   ,
  COUNTERBRIDGE_CODE_ID            ,
  SPOT_CODE_ID                     ,
  OVERNIGHT_CODE_ID                ,
  CASHRATE_CODE_ID                 ,
  INDICATIVES_CODE_ID              ,
  TABLE_CODE_ID                    ,
  SUMMARY_CODE_ID                  ,
  VOICE_SUMMARY_CODE_ID            ,
  DMECONNECTED_CODE_ID             ,
#ifdef DMEAPP
  MARKET_CUPOSL_ID                 ,
  LIMITES_INSTITUCION_ID           ,
  PENDIENTES_COBRANZA_ID           ,
  CONFIG_CUPO_CODE_ID              , 
  MARKETSPOTPRICES_CODE_ID         ,
#endif
  // Tables mantained in DS side and  DME. 
  MARKET_PU_INSCRITOS_DME_ID       ,
  // Tables not mantained by DME but published to DGI
  BRANCHES_CODE_ID                 ,
  BRANCHESRFQ_CODE_ID              ,
  USER_CODE_ID                     ,
  INCREMENTS_CODE_ID               ,
  CURRENCY_CODE_ID                 ,
  MARKETS_TABLE_CODE_ID            ,
  CCYPAIR_CODE_ID                  ,
  CONNDMSERVER_CODE_ID             ,
  HOLIDAY_CODE_ID                  ,
  RFQREQUEST_CODE_ID               ,
  TERM_CODE_ID                     ,
  STATISTICS_ETC_CODE_ID           ,
  // Tables not mantained by DME and not published
  DATAMANAGER_LIST_ID              ,
  INSTIT_CODE_ID                   ,
  CONNECTED_USERS_CODE_ID          ,
  INDICATIVESMARK_CODE_ID          ,
  SUPERGROUP_CODE_ID               ,
  VOICE_TERM_CODE_ID               ,
  STATISTICS_CODE_ID               ,
  // Tables not mantained by DME and published
  MARKET_BUYAVERPRICE_ID           ,
  // Tables not mantained and not used by DME
  PRIMARY_USERS_CODE_ID            ,
  CITIES_CODE_ID                   ,
  MARKETS_CODE_ID                  ,
  MONEYS_CODE_ID                   ,
  COUNTRIES_CODE_ID                ,
#ifdef DMEAPP
  INSTPERM_CODE_ID                 ,
#endif
  VOICE_TERM_GROUP_CODE_ID         ,
  VOICE_GROUP_CODE_ID              ,
  FIXEDINCOMEMARKETS_TABLE_ID      ,
  HISTORYTRANSDAYMEM_CODE_ID       ,
  RATES_TABLE_ID                   ,
  COBRANZAS_CODE_ID                ,  
#ifdef DMEAPP
  COMPART_CUPO_CODE_ID             , 
  FACTORES_RIESGO_ID               , 
#endif
  INDICATIVESMARK_DME_CODE_ID      ,
#ifdef GDIAPP
  SUMMARYRF_CODE_ID                ,
  SETTD_VOICE_SUMMARY_CODE_ID      ,
  HISTORYIC_CODE_ID                ,
#endif
#ifdef DMEAPP
  MARKET_CUPOSXPL_ID               ,
  MARKET_CUPOSXDIA_ID              , 
  TRADING_CODE_ID                  ,
#endif
  MAX_CODE_ID
};

class CTableBase;

typedef struct TABLE_ID_EQUIVALENCE
{
  int         type_id;
  int         code;
  int         table_source;
  int         publish_flag;
  int         start_protocol;
  CTableBase *Ptr_CTableBase;
} TABLE_ID_EQUIVALENCE;

#endif

#ifdef DAPI

enum
{
    ZERO_CODE_ID                ,                
    TABLE_CODE_ID               ,               
    VOICE_SUMMARY_CODE_ID       ,       
    BRANCHES_CODE_ID            ,            
    USER_CODE_ID                ,                
    DATAMANAGER_LIST_ID         ,         
    INSTIT_CODE_ID              ,              
    SUPERGROUP_CODE_ID          ,          
    PRIMARY_USERS_CODE_ID       ,       
    CITIES_CODE_ID              ,                 
    MARKETS_CODE_ID             ,             
    MONEYS_CODE_ID              ,              
    COUNTRIES_CODE_ID           ,           
    INSTPERM_CODE_ID            ,            
    VOICE_GROUP_CODE_ID         ,         
    FIXEDINCOMEMARKETS_TABLE_ID , 
    HISTORYTRANSDAYTERM_CODE_ID , 
    RATES_TABLE_ID              ,              
    EMISORES_CODE_ID            ,
    COMITENTES_CODE_ID          ,          
    FIDUCIARIOS_CODE_ID         ,         
    EMPLOYEE_CODE_ID            ,
    ESQUEMA_CODE_ID             ,
    SUBCOMITENTE_CODE_ID        ,
    MAX_CODE_ID                          
};

class CTableBase;

typedef struct TABLE_ID_EQUIVALENCE
{
  int         type_id;
  int         code;
  int         table_source;
  int         publish_flag;
  int         start_protocol;
  int         level;
  CTableBase *Ptr_CTableBase;
} TABLE_ID_EQUIVALENCE;

#endif // DAPI

#endif //__TBLEQUIV_H__

extern TABLE_ID_EQUIVALENCE G_table_id_equivalence[];

#ifdef TBL_GLOBAL_HERE
// Table code zero if there is more than one table code associated to the
// table code id that depends on the market. The table codes of this kind of
// tables are got from the market.conf file.
// If there is a table type that has a unique table code associated for all
// the markets, set it here.

#ifndef DAPI  

TABLE_ID_EQUIVALENCE G_table_id_equivalence[MAX_CODE_ID] = {
//type_id                         , code                        , table_source           , publish_flag 
{ZERO_CODE_ID                     , TBLNUM_NOT_USED             , TBLEQUIV_NO_SOURCE     , TABLE_NOT_PUBLISHED 
                                  , START_NO_PROTOCOL_USED      , NULL},
// Tables mantained by DME

#ifdef DMEAPP
{MARKET_INSTIT_NUM_ID             , TABLENUM_INSTIT_NUM         , TBLEQUIV_DS_DME_SOURCE , TABLE_PUBLISHED_TO_DS
                                  , START_WITH_CRC32_PROTOCOL   , &Glb_CGTInstNum_Table},
{MARKET_BRANCH_NUM_ID             , TABLENUM_BRANCH_NUM         , TBLEQUIV_DS_DME_SOURCE , TABLE_PUBLISHED_ALL
                                  , START_WITH_CRC32_PROTOCOL   , &Glb_CGTBranchNum_Table},
#else
{MARKET_INSTIT_NUM_ID             , TABLENUM_INSTIT_NUM         , TBLEQUIV_DME_SOURCE     , TABLE_PUBLISHED_TO_DS
                                  , START_NO_PROTOCOL_USED      , NULL},
{MARKET_BRANCH_NUM_ID             , TABLENUM_BRANCH_NUM         , TBLEQUIV_DME_SOURCE     , TABLE_PUBLISHED_TO_DS
                                  , START_NO_PROTOCOL_USED      , NULL},
#endif
{MARKET_CREDIT_ADMINISTRATION_ID  , TBLNUM_DYNAMIC_CODE         , TBLEQUIV_DME_SOURCE    , TABLE_PUBLISHED_TO_DS
                                  , START_NO_PROTOCOL_USED      , NULL},
{MARKET_CUPOS_ID                  , TBLNUM_DYNAMIC_CODE         , TBLEQUIV_DME_SOURCE    , TABLE_PUBLISHED_TO_DS
                                  , START_NO_PROTOCOL_USED      , NULL},
{MARKET_NAMECHECK_TERM_ID         , TBLNUM_DYNAMIC_CODE         , TBLEQUIV_DME_SOURCE    , TABLE_PUBLISHED_TO_DS
                                  , START_NO_PROTOCOL_USED      , NULL},
{BRIDGE_CODE_ID                   , TABLENUM_BRIDGE             , TBLEQUIV_DME_SOURCE    , TABLE_PUBLISHED_TO_DS
                                  , START_NO_PROTOCOL_USED      , NULL},
{COUNTERBRIDGE_CODE_ID            , TABLENUM_COUNTERBRIDGE      , TBLEQUIV_DME_SOURCE    , TABLE_PUBLISHED_TO_DS
                                  , START_NO_PROTOCOL_USED      , NULL},
{SPOT_CODE_ID                     , TABLENUM_SPOT               , TBLEQUIV_DME_SOURCE    , TABLE_PUBLISHED_ALL
                                  , START_NO_PROTOCOL_USED      , NULL},
{OVERNIGHT_CODE_ID                , TABLENUM_OVERNIGHT          , TBLEQUIV_DME_SOURCE    , TABLE_PUBLISHED_TO_DGI
                                  , START_NO_PROTOCOL_USED      , NULL},
{CASHRATE_CODE_ID                 , TABLENUM_CASHRATE           , TBLEQUIV_DME_SOURCE    , TABLE_PUBLISHED_ALL
                                  , START_NO_PROTOCOL_USED      , NULL},
{INDICATIVES_CODE_ID              , TBLNUM_DYNAMIC_CODE         , TBLEQUIV_DME_SOURCE    , TABLE_PUBLISHED_TO_DS
                                  , START_NO_PROTOCOL_USED      , NULL},
#ifdef GDIAPP
{TABLE_CODE_ID                    , TBLNUM_DYNAMIC_CODE         , TBLEQUIV_DS_GDI_SOURCE , TABLE_PUBLISHED_TO_DS
                                  , START_NO_PROTOCOL_USED      , NULL},
#else
{TABLE_CODE_ID                    , TBLNUM_DYNAMIC_CODE         , TBLEQUIV_DME_SOURCE    , TABLE_PUBLISHED_ALL
                                  , START_NO_PROTOCOL_USED      , NULL},
#endif
{SUMMARY_CODE_ID                  , TBLNUM_DYNAMIC_CODE         , TBLEQUIV_DME_SOURCE    , TABLE_PUBLISHED_ALL
                                  , START_NO_PROTOCOL_USED      , NULL},
#ifndef GDIAPP
{VOICE_SUMMARY_CODE_ID            , TBLNUM_DYNAMIC_CODE         , TBLEQUIV_DME_SOURCE    , TABLE_PUBLISHED_ALL
                                  , START_NO_PROTOCOL_USED      , NULL},
#else
{VOICE_SUMMARY_CODE_ID            , TBLNUM_DYNAMIC_CODE         , TBLEQUIV_DS_SOURCE     , TABLE_NOT_PUBLISHED
                                  , START_NO_PROTOCOL_USED      , NULL},

#endif 
{DMECONNECTED_CODE_ID             , TABLENUM_DMECONNECTED       , TBLEQUIV_DME_SOURCE    , TABLE_PUBLISHED_TO_DS
                                  , START_NO_PROTOCOL_USED      , NULL},
#ifdef DMEAPP
{MARKET_CUPOSL_ID                 , TABLENUM_CUPOSL             , TBLEQUIV_DS_DME_SOURCE , TABLE_PUBLISHED_TO_DS_AND_FROM_DS
                                  , START_WITH_CRC32_PROTOCOL   , &Glb_CuposL_Table},
{LIMITES_INSTITUCION_ID           , TABLENUM_LIMINSTIT          , TBLEQUIV_DS_DME_SOURCE , TABLE_PUBLISHED_TO_DS_AND_FROM_DS
                                  , START_WITH_CRC32_PROTOCOL   , &Glb_LimInstit_Table},
{PENDIENTES_COBRANZA_ID           , TABLENUM_PENDCOBRANZAS      , TBLEQUIV_DS_DME_SOURCE , TABLE_PUBLISHED_TO_DS_AND_FROM_DS
                                  , START_WITH_CRC32_PROTOCOL   , &Glb_PendCobranzas_Table},
{CONFIG_CUPO_CODE_ID              , TABLENUM_CONFIG_CUPO        , TBLEQUIV_DS_DME_SOURCE , TABLE_PUBLISHED_TO_DS_AND_FROM_DS
                                  , START_WITH_CRC32_PROTOCOL   , &Glb_ConfCupos_Table},
{MARKETSPOTPRICES_CODE_ID         , TABLENUM_MARKETSPOTPRICES   , TBLEQUIV_DS_DME_SOURCE , TABLE_PUBLISHED_TO_DS_AND_FROM_DS
                                  , START_NO_PROTOCOL_USED      , &Glb_MarketSpotPrices_Table},
#endif
// Tables mantained in DS side and DME.
{MARKET_PU_INSCRITOS_DME_ID       , TABLENUM_PUINSDME           , TBLEQUIV_DS_DME_SOURCE , TABLE_PUBLISHED_ALL
                                  , START_NO_PROTOCOL_USED      , NULL},
// Tables not mantained by DME but published to DGI
{BRANCHES_CODE_ID                 , TABLENUM_BRANCHES           , TBLEQUIV_DS_SOURCE     , TABLE_PUBLISHED_TO_DGI
                                  , START_NO_PROTOCOL_USED      , NULL},
{BRANCHESRFQ_CODE_ID              , TABLENUM_BRANCHESRFQS       , TBLEQUIV_DS_SOURCE     , TABLE_PUBLISHED_TO_DGI
                                  , START_NO_PROTOCOL_USED      , NULL},
{USER_CODE_ID                     , TABLENUM_USER               , TBLEQUIV_DS_SOURCE     , TABLE_PUBLISHED_TO_DGI
                                  , START_NO_PROTOCOL_USED      , NULL},
{INCREMENTS_CODE_ID               , TABLENUM_INCREMENTS         , TBLEQUIV_DS_SOURCE     , TABLE_PUBLISHED_TO_DGI
                                  , START_NO_PROTOCOL_USED      , NULL},
{CURRENCY_CODE_ID                 , TABLENUM_CURRENCY           , TBLEQUIV_DS_SOURCE     , TABLE_PUBLISHED_TO_DGI
                                  , START_NO_PROTOCOL_USED      , NULL},
{MARKETS_TABLE_CODE_ID            , TABLENUM_MARKETS            , TBLEQUIV_DS_SOURCE     , TABLE_PUBLISHED_TO_DGI
                                  , START_NO_PROTOCOL_USED      , NULL},
{CCYPAIR_CODE_ID                  , TABLENUM_CCYPAIR            , TBLEQUIV_DS_SOURCE     , TABLE_PUBLISHED_TO_DGI
                                  , START_NO_PROTOCOL_USED      , NULL},
{CONNDMSERVER_CODE_ID             , TABLENUM_CONNDMSERVER       , TBLEQUIV_DS_SOURCE     , TABLE_PUBLISHED_TO_DGI
                                  , START_NO_PROTOCOL_USED      , NULL},
{HOLIDAY_CODE_ID                  , TABLENUM_HOLIDAY            , TBLEQUIV_DS_SOURCE     , TABLE_PUBLISHED_TO_DGI
                                  , START_NO_PROTOCOL_USED      , NULL},
{RFQREQUEST_CODE_ID               , TABLENUM_RFQREQUEST         , TBLEQUIV_DS_SOURCE     , TABLE_PUBLISHED_TO_DGI
                                  , START_NO_PROTOCOL_USED      , NULL},
{TERM_CODE_ID                     , TBLNUM_DYNAMIC_CODE         , TBLEQUIV_DS_SOURCE     , TABLE_PUBLISHED_TO_DGI
                                  , START_NO_PROTOCOL_USED      , NULL},
{STATISTICS_ETC_CODE_ID           , TBLNUM_DYNAMIC_CODE         , TBLEQUIV_DS_SOURCE     , TABLE_PUBLISHED_TO_DGI
                                  , START_NO_PROTOCOL_USED      , NULL},
// Tables not mantained by DME and not published
{DATAMANAGER_LIST_ID              , TABLENUM_DATAMANAGERLIST    , TBLEQUIV_DS_SOURCE     , TABLE_NOT_PUBLISHED
                                  , START_NO_PROTOCOL_USED      , NULL},
{INSTIT_CODE_ID                   , TABLENUM_INSTITUTIONS       , TBLEQUIV_DS_SOURCE     , TABLE_NOT_PUBLISHED
                                  , START_NO_PROTOCOL_USED      , NULL},
{CONNECTED_USERS_CODE_ID          , TABLENUM_CONNUSER           , TBLEQUIV_DS_SOURCE     , TABLE_NOT_PUBLISHED
                                  , START_NO_PROTOCOL_USED      , NULL},
{INDICATIVESMARK_CODE_ID          , TABLENUM_INDICATIVESMARK    , TBLEQUIV_DS_SOURCE     , TABLE_NOT_PUBLISHED
                                  , START_NO_PROTOCOL_USED      , NULL},
{SUPERGROUP_CODE_ID               , TABLENUM_SUPERGROUP         , TBLEQUIV_DS_SOURCE     , TABLE_NOT_PUBLISHED
                                  , START_NO_PROTOCOL_USED      , NULL},
{VOICE_TERM_CODE_ID               , TABLENUM_VOICE_TERM         , TBLEQUIV_DS_SOURCE     , TABLE_NOT_PUBLISHED
                                  , START_NO_PROTOCOL_USED      , NULL},
{STATISTICS_CODE_ID               , TBLNUM_DYNAMIC_CODE         , TBLEQUIV_DS_SOURCE     , TABLE_NOT_PUBLISHED
                                  , START_NO_PROTOCOL_USED      , NULL},
// Tables not mantained by DME and published
{MARKET_BUYAVERPRICE_ID           , TBLNUM_DYNAMIC_CODE         , TBLEQUIV_DS_SOURCE     , TABLE_NOT_PUBLISHED
                                  , START_NO_PROTOCOL_USED      , NULL},
// Tables not mantained and not used by DME
{PRIMARY_USERS_CODE_ID            , TBLNUM_DYNAMIC_CODE         , TBLEQUIV_DS_SOURCE     , TABLE_NOT_PUBLISHED
                                  , START_NO_PROTOCOL_USED      , NULL},
{CITIES_CODE_ID                   , TBLNUM_DYNAMIC_CODE         , TBLEQUIV_DS_SOURCE     , TABLE_NOT_PUBLISHED
                                  , START_NO_PROTOCOL_USED      , NULL},
{MARKETS_CODE_ID                  , TBLNUM_DYNAMIC_CODE         , TBLEQUIV_DS_SOURCE     , TABLE_NOT_PUBLISHED
                                  , START_NO_PROTOCOL_USED      , NULL},
{MONEYS_CODE_ID                   , TBLNUM_DYNAMIC_CODE         , TBLEQUIV_DS_SOURCE     , TABLE_NOT_PUBLISHED
                                  , START_NO_PROTOCOL_USED      , NULL},
{COUNTRIES_CODE_ID                , TBLNUM_DYNAMIC_CODE         , TBLEQUIV_DS_SOURCE     , TABLE_NOT_PUBLISHED
                                  , START_NO_PROTOCOL_USED      , NULL},
#if DMEAPP
{INSTPERM_CODE_ID                 , TABLENUM_INSTPERM           , TBLEQUIV_DS_SOURCE     , TABLE_NOT_PUBLISHED
                                  , START_NO_PROTOCOL_USED      , NULL},
#endif
{VOICE_TERM_GROUP_CODE_ID         , TBLNUM_DYNAMIC_CODE         , TBLEQUIV_DS_SOURCE     , TABLE_NOT_PUBLISHED
                                  , START_NO_PROTOCOL_USED      , NULL},
#ifndef GDIAPP
{VOICE_GROUP_CODE_ID              , TABLENUM_VOICEGROUP         , TBLEQUIV_DS_SOURCE     , TABLE_NOT_PUBLISHED
                                  , START_NO_PROTOCOL_USED      , NULL},
#endif
#ifdef GDIAPP
{VOICE_GROUP_CODE_ID              , TABLENUM_VOICEGROUP         , TBLEQUIV_DS_SOURCE     , TABLE_NOT_PUBLISHED
                                  , START_NO_PROTOCOL_USED      , NULL},
#endif 
{FIXEDINCOMEMARKETS_TABLE_ID      , TBLNUM_DYNAMIC_CODE         , TBLEQUIV_DS_SOURCE     , TABLE_NOT_PUBLISHED
                                  , START_NO_PROTOCOL_USED      , NULL},

{HISTORYTRANSDAYMEM_CODE_ID       , TABLENUM_HISTORYTRANSDAYMEM , TBLEQUIV_DS_SOURCE     , TABLE_NOT_PUBLISHED
                                  , START_NO_PROTOCOL_USED      , NULL},

{RATES_TABLE_ID                   , TBLNUM_DYNAMIC_CODE         , TBLEQUIV_DS_SOURCE     , TABLE_NOT_PUBLISHED
                                  , START_NO_PROTOCOL_USED      , NULL},
{COBRANZAS_CODE_ID                , TABLENUM_COBRANZAS          , TBLEQUIV_DS_SOURCE     , TABLE_NOT_PUBLISHED
                                  , START_NO_PROTOCOL_USED      , NULL},
#ifdef DMEAPP
{COMPART_CUPO_CODE_ID             , TABLENUM_COMPART_CUPO       , TBLEQUIV_DS_SOURCE     , TABLE_NOT_PUBLISHED
                                  , START_WITH_CRC32_PROTOCOL   , &Glb_CompartCupos_Table},
{FACTORES_RIESGO_ID               , TABLENUM_FACTRIESGO         , TBLEQUIV_DS_DME_SOURCE , TABLE_PUBLISHED_TO_DS_AND_FROM_DS
                                  , START_WITH_CRC32_PROTOCOL   , &Glb_FactRiesgo_Table},
#endif
{INDICATIVESMARK_DME_CODE_ID      , TBLNUM_DYNAMIC_CODE         , TBLEQUIV_DME_SOURCE    , TABLE_NOT_PUBLISHED
                                  , START_NO_PROTOCOL_USED      , NULL},

#ifdef GDIAPP
{SUMMARYRF_CODE_ID                   , TABLENUM_SUMMARYRF          , TBLEQUIV_GDI_SOURCE    , TABLE_PUBLISHED_TO_DS
                                  , START_NO_PROTOCOL_USED      , NULL},                                  
{SETTD_VOICE_SUMMARY_CODE_ID      , TBLNUM_DYNAMIC_CODE         , TBLEQUIV_DS_SOURCE     , TABLE_NOT_PUBLISHED
                                  , START_NO_PROTOCOL_USED      , NULL},  
{HISTORYIC_CODE_ID                , TABLENUM_HISTORY_IC         , TBLEQUIV_DS_SOURCE     , TABLE_NOT_PUBLISHED
                                  , START_NO_PROTOCOL_USED      , NULL},  
#endif
#ifdef DMEAPP
{MARKET_CUPOSXPL_ID               , TABLENUM_CUPOSXPL           , TBLEQUIV_DS_DME_SOURCE , TABLE_PUBLISHED_TO_DS_AND_FROM_DS
                                  , START_WITH_CRC32_PROTOCOL   , &Glb_CuposXPL_Table},

{MARKET_CUPOSXDIA_ID              , TABLENUM_CUPOSXDIA          , TBLEQUIV_DS_DME_SOURCE , TABLE_PUBLISHED_TO_DS
                                  , START_WITH_CRC32_PROTOCOL   , &Glb_CuposXDIA_Table},
{TRADING_CODE_ID                  , TBLNUM_DYNAMIC_CODE         , TBLEQUIV_DME_SOURCE    , TABLE_PUBLISHED_TO_DS
                                  , START_NO_PROTOCOL_USED      , NULL},
#endif
//{MAX_CODE_ID                    , 0                           , TBLEQUIV_NO_SOURCE     , TABLE_NOT_PUBLISHED
//                                , START_DEFAULT_PROTOCOL      , NULL},
};
#endif


#ifdef DAPI

TABLE_ID_EQUIVALENCE G_table_id_equivalence[MAX_CODE_ID] = {
//type_id                         , code                        , table_source           , publish_flag 
{ZERO_CODE_ID                     , TBLNUM_NOT_USED             , TBLEQUIV_NO_SOURCE     , TABLE_NOT_PUBLISHED 
                                  , START_NO_PROTOCOL_USED      , 0                      , NULL},

{TABLE_CODE_ID                    , TBLNUM_DYNAMIC_CODE         , TBLEQUIV_DS_SOURCE     , TABLE_NOT_PUBLISHED | TABLE_USE_OWN_UNIQUE_IDX
                                  , START_NO_PROTOCOL_USED      , SECOND_LEVEL_TABLE     , &Glb_Order_Table},
                                  
{VOICE_SUMMARY_CODE_ID            , TBLNUM_DYNAMIC_CODE         , TBLEQUIV_DS_SOURCE     , TABLE_NOT_PUBLISHED
                                  , START_NO_PROTOCOL_USED      , FIRST_LEVEL_TABLE      , &Glb_VoiceSummary_Table},
                                  
{BRANCHES_CODE_ID                 , BRANCHES_TABLE_CODE         , TBLEQUIV_DS_SOURCE     , TABLE_NOT_PUBLISHED | TABLE_NO_LOG_ERRORS_INSERTION_MODIFICATION
                                  , START_WITH_CRC32_PROTOCOL   , FIRST_LEVEL_TABLE      , &Glb_Branches_Table},
                                  
{USER_CODE_ID                     , TBLNUM_DYNAMIC_CODE         , TBLEQUIV_DS_SOURCE     , TABLE_NOT_PUBLISHED 
                                  , START_NO_PROTOCOL_USED      , FIRST_LEVEL_TABLE      , &Glb_User_Table},
                                  
{DATAMANAGER_LIST_ID              , TABLENUM_DATAMANAGERLIST    , TBLEQUIV_DS_SOURCE     , TABLE_NOT_PUBLISHED
                                  , START_NO_PROTOCOL_USED      , FIRST_LEVEL_TABLE      , NULL},

{INSTIT_CODE_ID                   , INSTIT_TABLE_CODE           , TBLEQUIV_DS_SOURCE     , TABLE_NOT_PUBLISHED | TABLE_NO_LOG_ERRORS_INSERTION_MODIFICATION
                                  , START_WITH_CRC32_PROTOCOL   , FIRST_LEVEL_TABLE      , &Glb_Instit_Table},

{SUPERGROUP_CODE_ID               , SUPERGROUP_TABLE_CODE       , TBLEQUIV_DS_SOURCE     , TABLE_NOT_PUBLISHED | TABLE_NO_LOG_ERRORS_INSERTION_MODIFICATION
                                  , START_WITH_CRC32_PROTOCOL   , FIRST_LEVEL_TABLE      , &Glb_IntSuperGroup_Table},

{PRIMARY_USERS_CODE_ID            , PRIMARY_USERS_TABLE_CODE    , TBLEQUIV_DS_SOURCE     , TABLE_NOT_PUBLISHED | TABLE_NO_LOG_ERRORS_INSERTION_MODIFICATION
                                  , START_WITH_CRC32_PROTOCOL   , FIRST_LEVEL_TABLE      , &Glb_Primaryuser_Table},
                                  
{CITIES_CODE_ID                   , CITIES_TABLE_CODE           , TBLEQUIV_DS_SOURCE     , TABLE_NOT_PUBLISHED | TABLE_NO_LOG_ERRORS_INSERTION_MODIFICATION
                                  , START_WITH_CRC32_PROTOCOL   , FIRST_LEVEL_TABLE      , &Glb_City_Table},

{MARKETS_CODE_ID                  , TBLNUM_DYNAMIC_CODE         , TBLEQUIV_DS_SOURCE     , TABLE_NOT_PUBLISHED
                                  , START_NO_PROTOCOL_USED      , FIRST_LEVEL_TABLE      , &Glb_Markets_Table},
                                  
{MONEYS_CODE_ID                   , MONEYS_TABLE_CODE           , TBLEQUIV_DS_SOURCE     , TABLE_NOT_PUBLISHED | TABLE_NO_LOG_ERRORS_INSERTION_MODIFICATION
                                  , START_WITH_CRC32_PROTOCOL   , FIRST_LEVEL_TABLE      , &Glb_Money_Table},
                                  
{COUNTRIES_CODE_ID                , COUNTRIES_TABLE_CODE        , TBLEQUIV_DS_SOURCE     , TABLE_NOT_PUBLISHED | TABLE_NO_LOG_ERRORS_INSERTION_MODIFICATION
                                  , START_WITH_CRC32_PROTOCOL   , FIRST_LEVEL_TABLE      , &Glb_Country_Table},

{INSTPERM_CODE_ID                 , INSTPERM_TABLE_CODE         , TBLEQUIV_DS_SOURCE     , TABLE_NOT_PUBLISHED | TABLE_NO_LOG_ERRORS_INSERTION_MODIFICATION
                                  , START_WITH_CRC32_PROTOCOL   , FIRST_LEVEL_TABLE      , &Glb_InsPerm_Table},
                                  
{VOICE_GROUP_CODE_ID              , TBLNUM_DYNAMIC_CODE         , TBLEQUIV_DS_SOURCE     , TABLE_NOT_PUBLISHED | TABLE_NO_LOG_ERRORS_INSERTION_MODIFICATION
                                  , START_WITH_CRC32_PROTOCOL   , FIRST_LEVEL_TABLE      , &Glb_IntVoiceGroup_Table},

{FIXEDINCOMEMARKETS_TABLE_ID      , TBLNUM_DYNAMIC_CODE         , TBLEQUIV_DS_SOURCE     , TABLE_NOT_PUBLISHED | TABLE_NO_LOG_ERRORS_INSERTION_MODIFICATION
                                  , START_NO_PROTOCOL_USED      , FIRST_LEVEL_TABLE      , &Glb_IntFixedIncomeMarkets_Table},
                                  
{HISTORYTRANSDAYTERM_CODE_ID      , TBLNUM_DYNAMIC_CODE         , TBLEQUIV_DS_SOURCE     , TABLE_NOT_PUBLISHED | TABLE_NO_LOG_ERRORS_INSERTION_MODIFICATION
                                  , START_NO_PROTOCOL_USED      , SECOND_LEVEL_TABLE     , &Glb_IntHistoryTransDayTerm_Table},

{RATES_TABLE_ID                   , TBLNUM_DYNAMIC_CODE         , TBLEQUIV_DS_SOURCE     , TABLE_NOT_PUBLISHED | TABLE_NO_LOG_ERRORS_INSERTION_MODIFICATION
                                  , START_NO_PROTOCOL_USED      , SECOND_LEVEL_TABLE     , &Glb_IntRates_Table},
                                  
{EMISORES_CODE_ID                 , EMISORES_TABLE_CODE         , TBLEQUIV_DS_SOURCE     , TABLE_NOT_PUBLISHED | TABLE_NO_LOG_ERRORS_INSERTION_MODIFICATION
                                  , START_WITH_CRC32_PROTOCOL   , FIRST_LEVEL_TABLE      , &Glb_Emisores_Table},
                                  
{COMITENTES_CODE_ID               , COMITENTES_TABLE_CODE       , TBLEQUIV_DS_SOURCE     , TABLE_NOT_PUBLISHED | TABLE_NO_LOG_ERRORS_INSERTION_MODIFICATION
                                  , START_WITH_CRC32_PROTOCOL   , FIRST_LEVEL_TABLE      , &Glb_Comitentes_Table},

{FIDUCIARIOS_CODE_ID              , FIDUCIARIOS_TABLE_CODE      , TBLEQUIV_DS_SOURCE     , TABLE_NOT_PUBLISHED | TABLE_NO_LOG_ERRORS_INSERTION_MODIFICATION
                                  , START_WITH_CRC32_PROTOCOL   , FIRST_LEVEL_TABLE      , &Glb_Fiduciarios_Table},
                                  
{EMPLOYEE_CODE_ID                 , EMPLOYEE_TABLE_CODE         , TBLEQUIV_DS_SOURCE     , TABLE_NOT_PUBLISHED | TABLE_NO_LOG_ERRORS_INSERTION_MODIFICATION
                                  , START_WITH_CRC32_PROTOCOL   , FIRST_LEVEL_TABLE      , &Glb_Employee_Table},

{ESQUEMA_CODE_ID                  , ESQUEMA_TABLE_CODE          , TBLEQUIV_DS_SOURCE     , TABLE_NOT_PUBLISHED | TABLE_NO_LOG_ERRORS_INSERTION_MODIFICATION
                                  , START_WITH_CRC32_PROTOCOL   , FIRST_LEVEL_TABLE      , &Glb_Esquema_Table},

{SUBCOMITENTE_CODE_ID             , SUBCOMITENTE_TABLE_CODE     , TBLEQUIV_DS_SOURCE     , TABLE_NOT_PUBLISHED | TABLE_NO_LOG_ERRORS_INSERTION_MODIFICATION
                                  , START_WITH_CRC32_PROTOCOL   , FIRST_LEVEL_TABLE      , &Glb_SubComitente_Table},
};

#endif

#endif

//////////////////////////////////////////////////////////////////////////////
