
#ifndef __VECTOR_H__
#define __VECTOR_H__

#include <stdlib.h>

#define ARRAY_ELEM_IS_USED             -2
#define ARRAY_ELEM_LAST_AVAILABLE      -1

// this class handles the vector of objects of class T
template <class T>
class CVector
{
  int max_number;
  T **T_ptr;

public:
  CVector (int max);
 ~CVector (void);
  
  inline T &operator [](int item_num) {return *T_ptr[item_num];}
  inline T &operator ()(int item_num) {if (item_num >= max_number)
                                         Set_New_Limit (item_num*2);
                                       return *T_ptr[item_num];}
  void Set_New_Limit (int new_max_number);
};

template <class T>
class CVectorAt
{
  int first_free;
  T **T_ptr;
  int *At_ptr;
  char *T_type;

public:
  int cur_used;
  int max_used;
  int max_number;
  int last_free;

  CVectorAt (int max=20, char *type= NULL);
 ~CVectorAt (void);
  
  inline T &operator [](int item_num) {return *T_ptr[item_num];}
  inline T &operator ()(int item_num) {if (item_num >= max_number)
                                         Set_New_Limit (item_num*2);
                                       return *T_ptr[item_num];}
  void        Set_New_Limit (int new_max_number);

  int         Use_One   (void);
  void        Unrelease (int pos, int l_free);
  void        Release   (int pos);
  inline int  Is_Used   (int pos) { return At_ptr[pos] == ARRAY_ELEM_IS_USED; }
  inline int  Get_Element_Size(void) { return sizeof(T) + T_ptr[1]->Get_Rec_Length(); }
};

/////////////
// CVectorAt //
/////////////

template <class T>
CVectorAt<T>::CVectorAt (int max, char *type)
{ _SA_(0) 
  T_ptr      = NULL;
  At_ptr     = NULL;
  max_number = 0;
  max_used   = 0;
  cur_used   = 0;
  T_type     = type;
  first_free = ARRAY_ELEM_LAST_AVAILABLE;
  Set_New_Limit (max);
}

template <class T>
CVectorAt<T>:: ~CVectorAt (void)
{ _SA_(0) 
  int i;

  for (i=0; i< max_number; ++i)
    delete T_ptr[i];

  free (At_ptr);
  free (T_ptr);
}
  
template <class T>
void CVectorAt<T>::Set_New_Limit (int new_max_number)
{ _SA_(0) 
  int i;

  // check if the new limit is under the maximum value
  if (new_max_number <= max_number)
    return;

  T_ptr = (T **) realloc (T_ptr, new_max_number * sizeof(T *));
  for (i=max_number; i< new_max_number; ++i)
    T_ptr[i] = (T *) new T(T_type);

  At_ptr = (int *) realloc (At_ptr, new_max_number * sizeof (int));
  for (i=max_number; i< new_max_number-1; ++i)
    At_ptr[i] = i+1;

  if (first_free == ARRAY_ELEM_LAST_AVAILABLE)
    first_free        = max_number;
  else
    At_ptr[last_free] = max_number;

  last_free  = i;
  At_ptr[i]  = -1;

  max_number = new_max_number;
#if defined (DEBUG) || defined (_DEBUG)
  int tot;
  for (i=0,tot=0; i< max_number; ++i)
    if (At_ptr[i] == -1)
    {
      ++tot;
    }
  _ASSERT (tot == 1);

#endif
}

template <class T>
int CVectorAt<T>::Use_One (void)
{ _SA_(0) 
  int prev;
  prev = first_free;

  if (++cur_used > max_used)
    max_used = cur_used;
  first_free = At_ptr[prev];
  At_ptr[prev]  = ARRAY_ELEM_IS_USED;
  // check if this is the last
  if (first_free == ARRAY_ELEM_LAST_AVAILABLE)
  {
    // yes, increase the size
    Set_New_Limit (max_number * 2);
  }
  // check to have at least one more available
  // introduced to guarantee process when deleting and inserting from previous deleted record
  if (max_number - cur_used < 16)
    Set_New_Limit (max_number * 2);

#if defined (DEBUG_MEMORY)
#if defined (DEBUG) || defined (_DEBUG)
  int tot;
  int i;
  for (i=0,tot=0; i< max_number; ++i)
    if (At_ptr[i] == -1)
    {
      ++tot;
    }
  _ASSERT (tot == 1);
#endif
#endif
  return prev;
}

template <class T>
void CVectorAt<T>::Release (int pos)
{ _SA_(0) 
#if !USE_SOLARIS && !USE_LINUX
  _ASSERT (At_ptr [pos] == ARRAY_ELEM_IS_USED);
#endif

  --cur_used;

#if defined (DEBUG) || defined (_DEBUG)
  _ASSERT (At_ptr [last_free] == ARRAY_ELEM_LAST_AVAILABLE);
#endif

  At_ptr [last_free] = pos;                         // last_free was the last available, now this one es the next available
  At_ptr [pos]       = ARRAY_ELEM_LAST_AVAILABLE;   // this one is the last available
  last_free          = pos;
#if defined (DEBUG) || defined (_DEBUG)
  int tot;
  int i;
  for (i=0,tot=0; i< max_number; ++i)
    if (At_ptr[i] == -1)
    {
      ++tot;
    }
  _ASSERT (tot == 1);

#endif
}

// this is the undo of the release, this has to be done in the exact oposit order!
// to make this save the last_free value before to make the release, and use it
// as a second argumen to this method
template <class T>
void CVectorAt<T>::Unrelease (int pos, int l_free)
{ _SA_(0) 
#if !USE_SOLARIS && !USE_LINUX
  _ASSERT (At_ptr [pos] != ARRAY_ELEM_IS_USED);
#endif

  ++cur_used;
  At_ptr [l_free] = At_ptr [pos];
  if (At_ptr [l_free] == ARRAY_ELEM_LAST_AVAILABLE)
    last_free = l_free;

  At_ptr [pos]    = ARRAY_ELEM_IS_USED;

#if defined (DEBUG) || defined (_DEBUG)
  int tot;
  int i;
  for (i=0,tot=0; i< max_number; ++i)
    if (At_ptr[i] == -1)
    {
      ++tot;
    }
  _ASSERT (tot == 1);

#endif
}


#endif
